import java.applet.Applet;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * Simple drag-and-drop test applet.
 * Lists the flavors of data in a drag event.
 */
public class DragDropTest extends Applet implements DropTargetListener {
    private Label label;
    private TextArea textArea;

    /** Access to the applet as an application */
    public static void main(String[] args) {
        Frame f = new Frame("DragDropTest");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent we) {
                System.exit(0);
            }
        });
        f.setSize(640, 400);
        Applet a = new DragDropTest();
        f.add(a);
        a.init();
        a.start();
        f.setVisible(true);
    }
    public void init() {
        setLayout(new GridBagLayout());
        label = new Label("", Label.CENTER);
        resetLabel();
        Panel topPanel = new Panel();
        topPanel.add(label);
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        c.ipady = 5;
        add(topPanel, c);
        textArea = new TextArea("", 20, 60, TextArea.SCROLLBARS_VERTICAL_ONLY);
        textArea.setEditable(false);
        c.gridy = 1;
        c.ipady = 0;
        add(textArea, c);
        new DropTarget(topPanel, this);
    }
    public void dragOver(DropTargetDragEvent dtde) {
    }
    public void dragEnter(DropTargetDragEvent dtde) {
        considerDrag(dtde);
    }
    public void dragExit(DropTargetEvent dte) {
        resetLabel();
    }
    public void drop(DropTargetDropEvent dtde) {
        try {
            Transferable transferable = dtde.getTransferable();
            dtde.acceptDrop(DnDConstants.ACTION_COPY);
            logDrop(transferable);
            dtde.getDropTargetContext().dropComplete(true);
        } catch (RuntimeException re) {
            try { dtde.rejectDrop(); } catch (Exception e) {}
            throw re;
        } finally {
            resetLabel();
        }
    }
    public void dropActionChanged(DropTargetDragEvent dtde) {
        considerDrag(dtde);
    }
    private void setLabelText(String text) {
        label.setText("--> " + text + "<--");
    }
    private void resetLabel() {
        setLabelText("DRAG HERE");
    }
    private void considerDrag(DropTargetDragEvent dtde) {
        // Accept anything that has any data
        DataFlavor[] flavorList = dtde.getCurrentDataFlavors();
        if (flavorList == null || flavorList.length == 0) {
            dtde.rejectDrag();
            setLabelText("BAD DRAG?");
        } else {
            dtde.acceptDrag(DnDConstants.ACTION_COPY);
            setLabelText("LET GO!");
        }
    }
    private void logDrop(Transferable t) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append(new Date().toString());
            buffer.append("\n\n");
            if (t.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
                // Make extra effort for a file list so we can see it clearly
                buffer.append("Files dropped:\n\n");
                List fileList = (List) t.getTransferData(DataFlavor.javaFileListFlavor);
                for (Iterator i = fileList.iterator(); i.hasNext(); ) {
                    File file = (File) i.next();
                    buffer.append("  ");
                    buffer.append(file.toString());
                    buffer.append("\n");
                }
                buffer.append("\n");
            }
            buffer.append("Full data flavor list:\n\n");
            DataFlavor[] flavorList = t.getTransferDataFlavors();
            for (int i = 0; i < flavorList.length; i++) {
                DataFlavor df = flavorList[i];
                buffer.append("  flavorList[");
                buffer.append(i);
                buffer.append("]: ");
                buffer.append(df);
                buffer.append('\n');
                buffer.append("    ");
                Object transferData = t.getTransferData(df);
                buffer.append(transferData);
                buffer.append('\n');
            }
            textArea.setText(buffer.toString());
            textArea.setCaretPosition(0);
        } catch (UnsupportedFlavorException e) {
            throw new RuntimeException(e); // already checked
        } catch (IOException e) {
            throw new RuntimeException(e); // shouldn't happen?
        }
    }
}
